@@ -1,4 +1,4 @@
-from typing import Any, List, Optional
+from typing import Any, List, Optional, Union
 
 import nextcord
 from nextcord.ext import commands
@@ -28,7 +28,7 @@ class MenuPagesBase(Menu):
     def __init__(self, source: PageSource, **kwargs):
         self._source = source
         self.current_page = 0
-        if hasattr(self, "__button_menu_pages__"):
+        if isinstance(self, ButtonMenu):
             ButtonMenu.__init__(self, **kwargs)
             return
         Menu.__init__(self, **kwargs)
@@ -54,9 +54,7 @@ class MenuPagesBase(Menu):
         """
 
         if not isinstance(source, PageSource):
-            raise TypeError(
-                "Expected {0!r} not {1.__class__!r}.".format(PageSource, source)
-            )
+            raise TypeError("Expected {0!r} not {1.__class__!r}.".format(PageSource, source))
 
         self._source = source
         self.current_page = 0
@@ -79,7 +77,7 @@ class MenuPagesBase(Menu):
         --------
         TypeError
             The return value of :meth:`PageSource.format_page` was not a
-            :class:`str`, :class:`nextcord.Embed`, :class:`List[nextcord.Embed]`,
+            :class:`str`, :class:`nextcord.Embed`, List[:class:`nextcord.Embed`],
             or :class:`dict`.
         """
         value: PageFormatType = await nextcord.utils.maybe_coroutine(
@@ -91,9 +89,7 @@ class MenuPagesBase(Menu):
             return {"content": value}
         elif isinstance(value, nextcord.Embed):
             return {"embed": value}
-        elif isinstance(value, list) and all(
-            isinstance(v, nextcord.Embed) for v in value
-        ):
+        elif isinstance(value, list) and all(isinstance(v, nextcord.Embed) for v in value):
             return {"embeds": value}
         raise TypeError(
             "Expected {0!r} not {1.__class__!r}.".format(
@@ -109,11 +105,12 @@ class MenuPagesBase(Menu):
         page = await self._source.get_page(page_number)
         self.current_page = page_number
         kwargs = await self._get_kwargs_from_page(page)
+        assert self.message is not None, "Cannot show page without a message."
         await self.message.edit(**kwargs)
 
     async def send_initial_message(
         self, ctx: commands.Context, channel: nextcord.abc.Messageable
-    ) -> nextcord.Message:
+    ) -> Union[nextcord.Message, nextcord.PartialInteractionMessage]:
         """|coro|
 
         The default implementation of :meth:`Menu.send_initial_message`
@@ -125,10 +122,17 @@ class MenuPagesBase(Menu):
         kwargs = await self._get_kwargs_from_page(page)
         # filter out kwargs that are "None"
         kwargs = {k: v for k, v in kwargs.items() if v is not None}
+        # if we're not paginating, we can remove the pagination buttons
+        if not self._source.is_paginating():
+            await self.clear()
         # if there is an interaction, send an interaction response
         if self.interaction is not None:
             message = await self.interaction.send(ephemeral=self.ephemeral, **kwargs)
-            return message or await self.interaction.original_message()
+            # if we are adding reactions, we need the full interaction message
+            if isinstance(message, nextcord.PartialInteractionMessage) and self.buttons:
+                return await self.interaction.original_message()
+            # if we are only adding view buttons, we can return a PartialInteractionMessage or WebhookMessage
+            return message
         # otherwise, send the message using the channel
         return await channel.send(**kwargs)
 
@@ -149,9 +153,6 @@ class MenuPagesBase(Menu):
             wait=wait,
             ephemeral=ephemeral,
         )
-        # If we're not paginating, we can remove the pagination buttons
-        if not self._source.is_paginating():
-            await self.clear()
 
     async def show_checked_page(self, page_number: int):
         max_pages = self._source.get_max_pages()
@@ -188,7 +189,7 @@ class MenuPagesBase(Menu):
     async def go_to_last_page(self, payload=None):
         """go to the last page"""
         # The call here is safe because it's guarded by skip_if
-        await self.show_page(self._source.get_max_pages() - 1)
+        await self.show_page(self._source.get_max_pages() - 1)  # type: ignore
 
     async def stop_pages(self, payload=None):
         """stops the pagination session."""
@@ -208,7 +209,7 @@ class MenuPages(MenuPagesBase):
     def __init__(self, source: PageSource, **kwargs):
         super().__init__(source, **kwargs)
         # skip adding buttons if inherit_buttons=False was passed to metaclass
-        if not self.__inherit_buttons__:
+        if not self.__inherit_buttons__:  # type: ignore
             return
         # add pagination reaction buttons
         self.add_button(
@@ -219,9 +220,7 @@ class MenuPages(MenuPagesBase):
                 skip_if=self._skip_double_triangle_buttons,
             ),
         )
-        self.add_button(
-            Button(self.PREVIOUS_PAGE, self.go_to_previous_page, position=First(1))
-        )
+        self.add_button(Button(self.PREVIOUS_PAGE, self.go_to_previous_page, position=First(1)))
         self.add_button(
             Button(self.NEXT_PAGE, self.go_to_next_page, position=Last(0)),
         )
@@ -238,7 +237,7 @@ class MenuPages(MenuPagesBase):
         )
 
 
-class MenuPaginationButton(nextcord.ui.Button["MenuPaginationButton"]):
+class MenuPaginationButton(nextcord.ui.Button["ButtonMenuPages"]):
     """
     A custom button for pagination used by :class:`ButtonMenuPages`
     that runs pagination methods in the :meth:`MenuPaginationButton.callback`
@@ -261,19 +260,18 @@ class MenuPaginationButton(nextcord.ui.Button["MenuPaginationButton"]):
             return
 
         assert self.view is not None
-        view: ButtonMenuPages = self.view
 
         # change the current page
-        if str(self._emoji) == view.FIRST_PAGE:
-            await view.go_to_first_page()
-        elif str(self._emoji) == view.PREVIOUS_PAGE:
-            await view.go_to_previous_page()
-        elif str(self._emoji) == view.NEXT_PAGE:
-            await view.go_to_next_page()
-        elif str(self._emoji) == view.LAST_PAGE:
-            await view.go_to_last_page()
-        elif str(self._emoji) == view.STOP:
-            await view.stop_pages()
+        if str(self._emoji) == self.view.FIRST_PAGE:
+            await self.view.go_to_first_page()
+        elif str(self._emoji) == self.view.PREVIOUS_PAGE:
+            await self.view.go_to_previous_page()
+        elif str(self._emoji) == self.view.NEXT_PAGE:
+            await self.view.go_to_next_page()
+        elif str(self._emoji) == self.view.LAST_PAGE:
+            await self.view.go_to_last_page()
+        elif str(self._emoji) == self.view.STOP:
+            await self.view.stop_pages()
 
 
 class ButtonMenuPages(MenuPagesBase, ButtonMenu):
@@ -303,7 +301,7 @@ class ButtonMenuPages(MenuPagesBase, ButtonMenu):
             kwargs["disable_buttons_after"] = True
         super().__init__(source, **kwargs)
         # skip adding buttons if inherit_buttons=False was passed to metaclass or only one page
-        if not self.__inherit_buttons__ or not self.should_add_buttons():
+        if not self.__inherit_buttons__ or not self.should_add_buttons():  # type: ignore
             return
         # add buttons to the view
         pagination_emojis = (
@@ -342,7 +340,7 @@ class ButtonMenuPages(MenuPagesBase, ButtonMenu):
         --------
         TypeError
             The return value of :meth:`PageSource.format_page` was not a
-            :class:`str`, :class:`nextcord.Embed`, :class:`List[nextcord.Embed]`,
+            :class:`str`, :class:`nextcord.Embed`, List[:class:`nextcord.Embed`],
             or :class:`dict`.
         """
         kwargs = await super()._get_kwargs_from_page(page)
@@ -353,9 +351,8 @@ class ButtonMenuPages(MenuPagesBase, ButtonMenu):
         """
         Disables buttons that are unavailable to be pressed.
         """
-        children: List[MenuPaginationButton] = self.children
         max_pages = self._source.get_max_pages()
-        for child in children:
+        for child in self.children:
             if isinstance(child, nextcord.ui.Button):
                 if str(child.emoji) in (self.FIRST_PAGE, self.PREVIOUS_PAGE):
                     child.disabled = self.current_page == 0
