@@ -4,6 +4,7 @@ from typing import (
     Any,
     AsyncIterator,
     Callable,
+    Generic,
     List,
     NamedTuple,
     Optional,
@@ -12,7 +13,7 @@ from typing import (
     Union,
 )
 
-from .constants import PageFormatType, SendKwargsType
+from .constants import PageFormatType
 from .menus import Menu
 
 DataType = TypeVar("DataType")
@@ -123,7 +124,7 @@ class PageSource:
         as returning the ``embed`` keyword argument in :meth:`nextcord.Message.edit`
         and :meth:`nextcord.abc.Messageable.send`.
 
-        If this method returns a :class:`List[nextcord.Embed]` then it is interpreted
+        If this method returns a List[:class:`nextcord.Embed`] then it is interpreted
         as returning the ``embeds`` keyword argument in :meth:`nextcord.Message.edit`
         and :meth:`nextcord.abc.Messageable.send`.
 
@@ -141,13 +142,13 @@ class PageSource:
 
         Returns
         ---------
-        Union[:class:`str`, :class:`nextcord.Embed`, :class:`dict`]
+        Union[:class:`str`, :class:`nextcord.Embed`, List[:class:`nextcord.Embed`], :class:`dict`]
             See above.
         """
         raise NotImplementedError
 
 
-class ListPageSource(PageSource):
+class ListPageSource(PageSource, Generic[DataType]):
     """A data source for a sequence of items.
 
     This page source does not handle any sort of formatting, leaving it up
@@ -179,7 +180,7 @@ class ListPageSource(PageSource):
         """:class:`int`: The maximum number of pages required to paginate this sequence."""
         return self._max_pages
 
-    async def get_page(self, page_number: int) -> Union[DataType, List[DataType]]:
+    async def get_page(self, page_number: int) -> Union[DataType, Sequence[DataType]]:
         """Returns either a single element of the sequence or
         a slice of the sequence.
 
@@ -188,7 +189,7 @@ class ListPageSource(PageSource):
 
         Returns
         ---------
-        Union[Any, List[Any]]
+        Union[Any, Sequence[Any]]
             The data returned.
         """
         if self.per_page == 1:
@@ -197,18 +198,52 @@ class ListPageSource(PageSource):
             base = page_number * self.per_page
             return self.entries[base : base + self.per_page]
 
+    async def format_page(
+        self, menu: Menu, page: Union[DataType, List[DataType]]
+    ) -> PageFormatType:
+        """An abstract method to format the page.
+
+        This works similar to the :meth:`PageSource.format_page` except
+        the type of the ``page`` parameter is documented.
+
+        Parameters
+        ------------
+        menu: :class:`Menu`
+            The menu that wants to format this page.
+        page: Union[Any, List[Any]]
+            The page returned by :meth:`get_page`. This is either a single element
+            if :attr:`per_page` is set to ``1`` or a slice of the sequence otherwise.
+
+        Returns
+        ---------
+        Union[:class:`str`, :class:`nextcord.Embed`, List[:class:`nextcord.Embed`], :class:`dict`]
+            See :meth:`PageSource.format_page`.
+        """
+        raise NotImplementedError
+
 
 KeyType = TypeVar("KeyType")
 
 KeyFuncType = Callable[[DataType], KeyType]
 
 
-class _GroupByEntry(NamedTuple):
+class GroupByEntry(NamedTuple):
+    """Named tuple representing an entry returned by
+    :meth:`GroupByPageSource.get_page` in a :class:`GroupByPageSource`.
+
+    Attributes
+    ------------
+    key: Callable[[Any], Any]
+        A key of the :func:`itertools.groupby` function.
+    items: List[Any]
+        Slice of the paginated items within the group.
+    """
+
     key: KeyFuncType
-    items: DataType
+    items: List[Any]
 
 
-class GroupByPageSource(ListPageSource):
+class GroupByPageSource(ListPageSource, Generic[DataType]):
     """A data source for grouped by sequence of items.
 
     This inherits from :class:`ListPageSource`.
@@ -237,15 +272,10 @@ class GroupByPageSource(ListPageSource):
     """
 
     def __init__(
-        self,
-        entries: Sequence[DataType],
-        *,
-        key: KeyFuncType,
-        per_page: int,
-        sort: int = True
+        self, entries: Sequence[DataType], *, key: KeyFuncType, per_page: int, sort: int = True
     ):
         self.__entries = entries if not sort else sorted(entries, key=key)
-        nested: List[_GroupByEntry] = []
+        nested: List[GroupByEntry] = []
         self.nested_per_page = per_page
         for key_i, group_i in itertools.groupby(self.__entries, key=key):
             group_i = list(group_i)
@@ -255,35 +285,44 @@ class GroupByPageSource(ListPageSource):
 
             # Chunk the nested pages
             nested.extend(
-                _GroupByEntry(key=key_i, items=group_i[i : i + per_page])
+                GroupByEntry(key=key_i, items=group_i[i : i + per_page])
                 for i in range(0, size, per_page)
             )
 
         super().__init__(nested, per_page=1)
 
-    async def get_page(self, page_number: int) -> DataType:
+    async def get_page(self, page_number: int) -> GroupByEntry:
+        """Returns a :class:`GroupByEntry` with ``key``, representing the
+        key of the :func:`itertools.groupby` function, and ``items``,
+        representing a sequence of paginated items within that group.
+
+        Returns
+        ---------
+        GroupByEntry
+            The data returned.
+        """
         return self.entries[page_number]
 
-    async def format_page(self, menu: Menu, entry: _GroupByEntry) -> SendKwargsType:
+    async def format_page(self, menu: Menu, entry: GroupByEntry) -> PageFormatType:
         """An abstract method to format the page.
 
-        This works similar to the :meth:`ListPageSource.format_page` except
-        the return type of the ``entry`` parameter is documented.
+        This works similar to the :meth:`PageSource.format_page` except
+        the type of the ``entry`` parameter is documented.
 
         Parameters
         ------------
         menu: :class:`Menu`
             The menu that wants to format this page.
-        entry
-            A namedtuple with ``(key, items)`` representing the key of the
-            group by function and a sequence of paginated items within that
-            group.
+        entry: GroupByEntry
+            The page returned by :meth:`get_page`. This will be a
+            :class:`GroupByEntry` with ``key``, representing the key of the
+            :func:`itertools.groupby` function, and ``items``, representing
+            a sequence of paginated items within that group.
 
         Returns
         ---------
-        :class:`dict`
-            A dictionary representing keyword-arguments to pass to
-            the message related calls.
+        Union[:class:`str`, :class:`nextcord.Embed`, List[:class:`nextcord.Embed`], :class:`dict`]
+            See :meth:`PageSource.format_page`.
         """
         raise NotImplementedError
 
@@ -301,7 +340,7 @@ def _aiter(obj, *, _isasync=inspect.iscoroutinefunction):
     return async_iter
 
 
-class AsyncIteratorPageSource(PageSource):
+class AsyncIteratorPageSource(PageSource, Generic[DataType]):
     """A data source for data backed by an asynchronous iterator.
 
     This page source does not handle any sort of formatting, leaving it up
@@ -389,3 +428,26 @@ class AsyncIteratorPageSource(PageSource):
             return await self._get_single_page(page_number)
         else:
             return await self._get_page_range(page_number)
+
+    async def format_page(
+        self, menu: Menu, page: Union[DataType, List[DataType]]
+    ) -> PageFormatType:
+        """An abstract method to format the page.
+
+        This works similar to the :meth:`PageSource.format_page` except
+        the type of the ``page`` parameter is documented.
+
+        Parameters
+        ------------
+        menu: :class:`Menu`
+            The menu that wants to format this page.
+        page: Union[Any, List[Any]]
+            The page returned by :meth:`get_page`. This is either a single element
+            if :attr:`per_page` is set to ``1`` or a slice of the sequence otherwise.
+
+        Returns
+        ---------
+        Union[:class:`str`, :class:`nextcord.Embed`, List[:class:`nextcord.Embed`], :class:`dict`]
+            See :meth:`PageSource.format_page`.
+        """
+        raise NotImplementedError
